/* IPFormatToText.cmd:	Convert IPFormat output to more readable text
			Read named file or stdin.  Write stdout

   Copyright (c) 2001, 2003 Steven Levine and Associates, Inc.
   All rights reserved.

   This program is free software licensed under the terms of the GNU
   General Public License.  The GPL Software License can be found in
   gnugpl2.txt or at http://www.gnu.org/licenses/licenses.html#GPL

   $TLIB$: $ &(#) %n - Ver %v, %f $
   TLIB: $ $

   Revisions	07 Sep 01 SHL - Baseline
		19 Mar 02 SHL - Update help
		29 Mar 03 SHL - Update license

*/

signal on Error
signal on FAILURE name Error
signal on Halt
signal on NOTREADY name Error
signal on NOVALUE name Error
signal on SYNTAX name Error

call Initialize

Gbl.Version = '0.2'

Main:

  parse arg szArgs
  call ScanArgs szArgs
  drop szArgs

  if Gbl.aArgs.0 = 0 then do
    Gbl.aArgs.1 = 'STDIN'
    Gbl.aArgs.0 = 1
  end

  cLinesRead = 0
  cLinesWritten = 0
  fInPacket = 0
  szOut = ''
  szCR = x2c('0d')
  szLF = x2c('0a')
  szNoPrint = xrange('00'x, '1f'x) || xrange('80'x, 'ff'x)

  do iArg = 1 to Gbl.aArgs.0

    szInFile = Gbl.aArgs.iArg

    call lineout 'STDERR', 'IPFormatToText reading' szInFile

    if szInFile \= 'STDIN' then do

      if stream(szInFile, 'C', 'QUERY EXISTS') == '' then do
	call Fatal szInFile 'does not exist.'
      end

      call stream szInFile, 'C', 'OPEN READ'
    end

    drop szCondition

    do while lines(szInFile) \= 0

      cLinesRead = cLinesRead + 1

      call on NOTREADY name CatchError	/* Avoid death on missing NL */
      szLine = linein(szInFile)
      signal on NOTREADY name Error

      l = length(szLine)

      select
      when l = 0 then do
	if fInPacket then do
	  if szOut \= '' then do
	    call lineout ,szOut
	    cLinesWritten = cLinesWritten + 1
	  end
	  fInPacket = 0
	end
	call lineout ,''
	cLinesWritten = cLinesWritten + 1
      end
      when pos('----', szLine) = 1 &,
	   (pos('PACKET', szLine) \= 0 | pos('DATA', szLine) \= 0) then do
	fInPacket = 1
	szOut = ''
	call lineout ,szLine
	cLinesWritten = cLinesWritten + 1
      end
      when fInPacket then do
	szWork = strip(substr(szLine, 6, 50))
	do while szWork \== ''
	  ch = x2c(left(szWork, 2))
	  szWork = strip(substr(szWork, 3))
	  if pos(ch, szNoPrint) = 0 then do
	    szOut = szOut || ch
	    if length(szOut) >= 72 then do
	      call lineout ,szOut
	      szOut = ''
	      cLinesWritten = cLinesWritten + 1
	    end
	  end
	  else do
	    /* fixme to not ignore other non-printables */
	    if ch = szCR | ch = szLF then do
	      if length(szOut) > 0 then do
		call lineout ,szOut
		szOut = ''
		cLinesWritten = cLinesWritten + 1
	      end
	    end
	  end
	end /* while */
      end /* fInPacket */
      otherwise
	call lineout ,szLine
	cLinesWritten = cLinesWritten + 1
      end

    end /* while lines */

    if szInFile \= 'STDIN' then
      call stream szInFile, 'C', 'CLOSE'

  end /* iArgs */

  call lineout 'STDERR', 'Read' cLinesRead 'lines'

  exit

/* end main */

/*=== Initialize: Intialize globals ===*/

Initialize: procedure expose Gbl.

  call LoadFuncs

  /* Set: Gbl.szCmdName	*/

  parse source . . sz
  sz = filespec('N', sz)
  c = pos('.', sz)
  if c > 1 then
    sz = left(sz, c - 1)
  Gbl.szCmdName = ToLower(sz)

  return

/* end Initialize */

/*=== ScanArgs(argLine): scan command line arguments and switches ===*/

ScanArgs: procedure expose Gbl.

  /* Evaluate arguments - override
     Return Gbl.Opt. and Gbl.aArgs. etc.
  */

  parse arg szRest
  szRest = strip(szRest)

  /* Set defaults */
  Gbl.Opt.fDebug = 0
  Gbl.Opt.fVerbose = 0			/* Verbose messages */
  Gbl.aArgs.0 = 0			/* Init arg count */

  /* Prepare scanner */
  szSwCtl = ''				/* Switches that take args */
  szArg = ''				/* Current argument string */
  szSw = ''				/* Current switch list */
  fSwEnd = 0				/* End of switches */

  do while szRest \== '' | szArg \== '' | szSw \== ''

    if szArg == '' then do
      /* Buffer empty, refill */
      szQ = left(szRest, 1)		/* Remember quote */
      if \ verify(szQ,'''"', 'M') then do
	parse var szRest szArg szRest	/* Not quoted */
      end
      else do
	/* Arg is quoted */
	szArg = ''
	do forever
		/* Parse dropping quotes */
	  parse var szRest (szQ)szArg1(szQ) szRest
	  szArg = szArg || szArg1
		/* Check for escaped quote within quoted string (i.e. "" or '') */
	  if left(szRest, 1) \== szQ then
	    leave			/* No, done */
	  szArg = szArg || szQ		/* Append quote */
	  parse var szRest (szQ) szRest
	end /* do */
      end /* if quoted */
    end

    /* If switch buffer empty, refill */
    if szSw == '' then do
      if left(szArg, 1) == '-' then do
	if fSwEnd then
	  call Usage 'switch '''szArg''' unexpected'
	else if szArg == '--' then
	  fSwEnd = 1
	else
	  szSw = substr(szArg, 2)
	parse var szRest szArg szRest
      end
    end

    /* If switch in progress */
    if szSw \== '' then do
      sz = left(szSw, 1)		/* Next switch */
      szSw = substr(szSw, 2)		/* Drop from pending */
      if Gbl.Opt.fDebug then
	say '* Switch' sz
      /* Check switch requires argument */
      if pos(sz, szSwCtl) \= 0 then do
	if szSw \== '' then do
	  szOpt = szSw
	  szSw = ''
	end
	else if szArg \== '' & left(szArg, 1) \= '-' then do
	  szOpt = szArg
	  parse var szRest szArg szRest
	end
	else
	  call Usage 'Switch' sz 'requires argument'
	if Gbl.Opt.fDebug then
	  say '* Opt' szOpt
      end
      select
      when sz == 'd' then
	Gbl.Opt.fDebug = 1
      when sz == 'h' | sz == '?' then
	call UsageHelp
      when sz == 'V' then do
	say Gbl.szCmdName Gbl.Version
	exit
      end
      otherwise
	call Usage 'switch '''sz''' unexpected'
      end /* select */
    end /* if switch */

    /* If arg */
    else if szArg \== '' then do
      fSwEnd = 1			/* No more switches */
      if Gbl.Opt.fDebug then
	say '* Arg' szArg
      /* Got non switch arg */
      i = Gbl.aArgs.0 + 1
      Gbl.aArgs.i = szArg
      Gbl.aArgs.0 = i
      szArg = ''
    end

  end /* while szRest */

  return

/* end ScanArgs */

/*=== Usage(szMsg): Report Usage Error ===*/

Usage:
  parse arg szMsg
  say szMsg
  say 'Usage:' Gbl.szCmdName '[-h] [-V] [filename...]'
  exit 255

/* end Usage */

/*=== Usage: Display help ===*/

UsageHelp:

  say
  say 'Usage:' Gbl.szCmdName '[-h] [-V] [filename...]'
  say
  say ' -h        Show this message'
  say ' -V        Show version'
  say
  say ' filename  File containing ipformat output'
  say '           Default is to read stdin'
  say
  say ' Output is written to stdout'

  exit 255

/* end UsageHelp */

/*========================================================================== */
/*=== Standard functions - Delete unused.  Move modified above this mark === */
/*========================================================================== */

/*=== CatchError: Catch condition for user ===*/

CatchError:

  szCondition = condition('C')
  return

/* end CatchError */

/*=== Error: Trap ERROR, FAILURE etc. conditions ===*/

Error:

  /*=== Returns szCondition or Exits ===*/

  say
  parse source . . szThisCmd
  say condition('C') 'signaled at' SIGL 'of' szThisCmd
  drop szThisCmd
  say 'Source =' sourceline(SIGL)
  call SysSleep 2
  if condition('I') == 'CALL' then do
    szCondition = condition('C')
    say 'Returning'
    return
  end
  else do
    if /*fixme */ 0 then trace '?A'
    say 'Exiting'
    call SysSleep 2
    exit 255
  end

/* end Error */

/*=== Fatal: Report fatal error and exit ===*/

Fatal:

  parse arg szMsg

  say
  say Gbl.szCmdName':' szMsg

  call Beep 200, 300

  exit 254

/* end Fatal */

/*=== Halt: Trap HALT condition ===*/

Halt:

  /*=== Returns szCondition or Exits ===*/

  say
  parse source . . szThisCmd
  say condition('C') 'signaled at' SIGL 'of' szThisCmd
  drop szThisCmd
  say 'Source = ' sourceline(SIGL)
  call SysSleep 2
  if condition('I') == 'CALL' then do
    szCondition = condition('C')
    say 'Returning'
    return
  end
  else do
   say 'Exiting'
   exit
  end

/* end Halt */

/*=== LoadFuncs: Load fuctions ===*/

LoadFuncs:

  /* Add all Rexx functions */
  if RxFuncQuery('SysLoadFuncs') then do
    call RxFuncAdd 'SysLoadFuncs', 'REXXUTIL', 'SysLoadFuncs'
    if RESULT then do
      say 'Cannot load SysLoadFuncs'
      exit
    end
    call SysLoadFuncs
  end /* end do */

  return

/* end LoadFuncs */

/*=== ToLower(sz): Convert to lower case ===*/

ToLower: procedure

  parse arg sz
  return translate(sz, xrange('a', 'z'), xrange('A', 'Z'))

/* end ToLower */

/* The end */
